'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.dehydrate = dehydrate;
exports.hydrate = hydrate;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _rx = require('rx');

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _utils = require('./utils');

var assign = Object.assign;
var __DEV__ = process.env.NODE_ENV === 'development';
var debug = (0, _debug2['default'])('thundercats:translate');

function dehydrate(cat) {
  if (__DEV__) {
    (0, _invariant2['default'])((0, _utils.isKitten)(cat), 'dehydrate should get an instance of the cat, but got %s', cat);
  }

  return _rx.Observable.from(cat.stores.values())
  // store must have displayName and value
  .filter(function (store) {
    return !!(0, _utils.getNameOrNull)(store) && !!store.value;
  }).map(function (store) {
    return _defineProperty({}, (0, _utils.getName)(store), store.value);
  }).reduce(function (allDats, storeDats) {
    return assign(allDats, storeDats);
  }, {}).map(function (allDats) {
    return allDats;
  }).tapOnError(function (err) {
    debug('an error occurred while dehydrating stores', err);
  });
}

function hydrate(cat, storesState) {
  if (__DEV__) {
    (0, _invariant2['default'])((0, _utils.isKitten)(cat), 'dehydrate should get an instance of the cat, but got %s', cat);
  }

  return _rx.Observable.combineLatest([_rx.Observable.from(cat.stores.values()), _rx.Observable.just(storesState).tap((0, _utils.createObjectValidator)('hydrate should get objects but got %s'))], function (store, stateMap) {
    return {
      store: store,
      data: stateMap[(0, _utils.getNameOrNull)(store)]
    };
  })
  // filter out falsey data and non objects
  .filter(function (_ref2) {
    var data = _ref2.data;
    return data && typeof data === 'object';
  })
  // assign value to store
  .tap(function (_ref3) {
    var store = _ref3.store;
    var data = _ref3.data;

    debug('updating %s with value: ', (0, _utils.getName)(store), data);
    store.value = data;
  })
  // wait to run through all the stores
  ['do'](null, function (err) {
    return debug('deserialize encountered a err', err);
  }, function () {
    return debug('deserialize completed');
  }).last().map(function () {
    return cat;
  });
}